// Copyright © 2021 Charles Kerr. All rights reserved.
// Created on:  5/19/21

#ifndef DataCategory_hpp
#define DataCategory_hpp

#include <cstdint>
#include <string>
#include <map>
#include <memory>
#include <filesystem>
#include <tuple>



enum DEFINITIONCATEGORIES
{
	invalid_cat = -1 ,
	items_def	= 0,
	npc_def,
	create_def,
	regions_def,
	misc_def,
	skills_def,
	location_def,
	menus_def,
	spells_def,
	newbie_def,
	titles_def,
	advance_def,
	house_def,
	colors_def,
	spawn_def,
	html_def,
	race_def,
	weathab_def,
	hard_items_def,
	command_def,
	msgboard_def,
	carve_def,
	creatures_def,
	maps_def
};



#include "DataSection.hpp"



class DataCategory {
private:

	DEFINITIONCATEGORIES _mycategory ;
	std::filesystem::path _basedir ;
	std::map<std::string, std::shared_ptr<DataSection> > _sections;
	
	std::string retrieveSectionName(const std::string& data);
	std::shared_ptr<KeyValue> processLine(const std::string line,const std::string &filepath ="", int linenumber = 0);
	std::map<std::string,int> retreivePostFiles() ;
	std::tuple<std::string,std::string> parseKeyValue(const std::string& data,const std::string &sep = "=") ;
	std::tuple<int,int> convertToInt(const std::string& value, const std::string &sep = " ");

protected:
	
public:
	static const std::map<DEFINITIONCATEGORIES,std::string> directories ;

	DataCategory();
	DataCategory(const std::string &filepath);
	void loadData(const std::string &directory, DEFINITIONCATEGORIES category,const std::string &extension=".dfn") ;
	void loadFile(const std::string& filepath);

	std::shared_ptr<DataSection> section(const std::string& name);
	std::shared_ptr<DataSection> containsName(const std::string& name);
	
	std::size_t size() const ;
	std::map<std::string,std::shared_ptr<DataSection> > sections() const;
	
	DEFINITIONCATEGORIES category() const ;
	void setCategory(DEFINITIONCATEGORIES value);
	void clear() ;
	
};

#endif /* DataCategory_hpp */
