// Copyright © 2021 Charles Kerr. All rights reserved.
// Created on:  5/22/21

#include "KeyValue.hpp"

#include <cstdint>
#include <iostream>
#include <stdexcept>
//+++++++++++++++++++++++++++++++++++++++++++++++++++
//
//		Methods for KeyValue
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++
const std::map<std::string, DFNTAGS> KeyValue::_keyenum_mapping = {
	{"AC",				DFNTAG_AC},
	{"ADDMENUITEM",		 	DFNTAG_ADDMENUITEM},
	{"ADVOBJ",			 	DFNTAG_ADVOBJ},
	{"ALCHEMY",			 	DFNTAG_ALCHEMY},
	{"AMMO",				DFNTAG_AMMO},
	{"AMMOFX",			 	DFNTAG_AMMOFX},
	{"AMOUNT",			 	DFNTAG_AMOUNT},
	{"ANATOMY",			 	DFNTAG_ANATOMY},
	{"ANIMALLORE",		 	DFNTAG_ANIMALLORE},
	{"ARCHERY",			 	DFNTAG_ARCHERY},
	{"ARMSLORE",			DFNTAG_ARMSLORE},
	{"ATT",				DFNTAG_ATT},
	{"BACKPACK",			DFNTAG_BACKPACK},
	{"BASERANGE",		 	DFNTAG_BASERANGE},
	{"BEGGING",			 	DFNTAG_BEGGING},
	{"BLACKSMITHING",	 		DFNTAG_BLACKSMITHING},
	{"BOWCRAFT",			DFNTAG_BOWCRAFT},
	{"BUSHIDO",			 	DFNTAG_BUSHIDO},
	{"CAMPING",			 	DFNTAG_CAMPING},
	{"CARPENTRY",		 	DFNTAG_CARPENTRY},
	{"CARTOGRAPHY",		 	DFNTAG_CARTOGRAPHY},
	{"CARVE",			 	DFNTAG_CARVE},
	{"CHIVALRY",			DFNTAG_CHIVALRY},
	{"COLD",				DFNTAG_COLD},
	{"COLOR",			 	DFNTAG_COLOUR},
	{"COLOUR",			 	DFNTAG_COLOUR},
	{"COLORLIST",		 	DFNTAG_COLOURLIST},
	{"COLOURLIST",		 	DFNTAG_COLOURLIST},
	{"COLORMATCHHAIR",	 	DFNTAG_COLOURMATCHHAIR},
	{"COLOURMATCHHAIR",	 	DFNTAG_COLOURMATCHHAIR},
	{"COOKING",			 	DFNTAG_COOKING},
	{"CORPSE",			 	DFNTAG_CORPSE},
	{"CREATOR",			 	DFNTAG_CREATOR},
	{"CUSTOMSTRINGTAG",	 	DFNTAG_CUSTOMSTRINGTAG},
	{"CUSTOMINTTAG",		 	DFNTAG_CUSTOMINTTAG},
	{"DAMAGE",			 	DFNTAG_DAMAGE},
	{"DECAY",			 	DFNTAG_DECAY},
	{"DEF",				DFNTAG_DEF},
	{"DETECTINGHIDDEN",	 	DFNTAG_DETECTINGHIDDEN},
	{"DEX",				DFNTAG_DEX},
	{"DEXTERITY",		 	DFNTAG_DEX},
	{"DX",				DFNTAG_DEX},
	{"DEXADD",			 	DFNTAG_DEXADD},
	{"DX2",				DFNTAG_DEXADD},
	{"DIR",				DFNTAG_DIR},
	{"DIRECTION",		 	DFNTAG_DIR},
	{"DISPELLABLE",		 	DFNTAG_DISPELLABLE},
	{"DISABLED",			DFNTAG_DISABLED},
	{"DOORFLAG",			DFNTAG_DOORFLAG},
	{"DYE",				DFNTAG_DYE},
	{"DYEABLE",			 	DFNTAG_DYE},
	{"DYEHAIR",			 	DFNTAG_DYEHAIR},
	{"DYEBEARD",			DFNTAG_DYEBEARD},
	{"ELEMENTRESIST",	 		DFNTAG_ELEMENTRESIST},
	{"EMOTECOLOR",		 	DFNTAG_EMOTECOLOUR},
	{"EMOTECOLOUR",		 	DFNTAG_EMOTECOLOUR},
	{"ENTICEMENT",		 	DFNTAG_ENTICEMENT},
	{"EVALUATINGINTEL",	 	DFNTAG_EVALUATINGINTEL},
	{"FAME",				DFNTAG_FAME},
	{"FENCING",			 	DFNTAG_FENCING},
	{"FISHING",			 	DFNTAG_FISHING},
	{"FLAG",				DFNTAG_FLAG},
	{"FLEEAT",			 	DFNTAG_FLEEAT},
	{"FLEEINGSPEED",		 	DFNTAG_FLEEINGSPEED},
	{"FOCUS",			 	DFNTAG_FOCUS},
	{"FORENSICS",		 	DFNTAG_FORENSICS},
	{"FX1",				DFNTAG_FX1},
	{"FX2",				DFNTAG_FX2},
	{"FY1",				DFNTAG_FY1},
	{"FY2",				DFNTAG_FY2},
	{"FZ1",				DFNTAG_FZ1},
	{"FOOD",				DFNTAG_FOOD},
	{"GET",				DFNTAG_GET},
	{"GLOW",				DFNTAG_GLOW},
	{"GLOWBC",			 	DFNTAG_GLOWBC},
	{"GLOWTYPE",			DFNTAG_GLOWTYPE},
	{"GOLD",				DFNTAG_GOLD},
	{"GOOD",				DFNTAG_GOOD},
	{"HAIRCOLOR",		 	DFNTAG_HAIRCOLOUR},
	{"HAIRCOLOUR",		 	DFNTAG_HAIRCOLOUR},
	{"HEALING",			 	DFNTAG_HEALING},
	{"HEAT",				DFNTAG_HEAT},
	{"HERDING",			 	DFNTAG_HERDING},
	{"HIDAMAGE",			DFNTAG_HIDAMAGE},
	{"HIDING",			 	DFNTAG_HIDING},
	{"HP",				DFNTAG_HP},
	{"HPMAX",			 	DFNTAG_HPMAX},
	{"ID",				DFNTAG_ID},
	{"IMBUING",			 	DFNTAG_IMBUING},
	{"IN",				DFNTAG_INTELLIGENCE},
	{"INTELLIGENCE",		 	DFNTAG_INTELLIGENCE},
	{"INT",				DFNTAG_INTELLIGENCE},
	{"IN2",				DFNTAG_INTADD},
	{"INTADD",			 	DFNTAG_INTADD},
	{"INTERVAL",			DFNTAG_INTERVAL},
	{"INSCRIPTION",		 	DFNTAG_INSCRIPTION},
	{"EQUIPITEM",		 	DFNTAG_EQUIPITEM},
	{"ITEMLIST",			DFNTAG_ITEMLIST},
	{"ITEMID",			 	DFNTAG_ITEMID},
	{"KARMA",			 	DFNTAG_KARMA},
	{"KILLHAIR",			DFNTAG_KILLHAIR},
	{"KILLBEARD",		 	DFNTAG_KILLBEARD},
	{"KILLPACK",			DFNTAG_KILLPACK},
	{"LAYER",			 	DFNTAG_LAYER},
	{"LIGHT",			 	DFNTAG_LIGHT},
	{"LIGHTNING",		 	DFNTAG_LIGHTNING},
	{"LOCKPICKING",		 	DFNTAG_LOCKPICKING},
	{"LODAMAGE",			DFNTAG_LODAMAGE},
	{"LOOT",				DFNTAG_LOOT},
	{"LUMBERJACKING",	 		DFNTAG_LUMBERJACKING},
	{"MACEFIGHTING",		 	DFNTAG_MACEFIGHTING},
	{"MAGERY",			 	DFNTAG_MAGERY},
	{"MAGICRESISTANCE",	 	DFNTAG_MAGICRESISTANCE},
	{"MANA",				DFNTAG_MANA},
	{"MANAMAX",			 	DFNTAG_MANAMAX},
	{"MAXRANGE",			DFNTAG_MAXRANGE},
	{"MAXHP",			 	DFNTAG_MAXHP},
	{"MAXITEMS",			DFNTAG_MAXITEMS},
	{"MEDITATION",		 	DFNTAG_MEDITATION},
	{"MINING",			 	DFNTAG_MINING},
	{"MOVABLE",			 	DFNTAG_MOVABLE},
	{"MORE",				DFNTAG_MORE},
	{"MORE2",			 	DFNTAG_MORE2},
	{"MOREX",			 	DFNTAG_MOREX},
	{"MOREY",			 	DFNTAG_MOREY},
	{"MOREZ",			 	DFNTAG_MOREZ},
	{"MUSICIANSHIP",		 	DFNTAG_MUSICIANSHIP},
	{"MYSTICISM",		 	DFNTAG_MYSTICISM},
	{"NAME",				DFNTAG_NAME},
	{"NAME2",			 	DFNTAG_NAME2},
	{"NAMELIST",			DFNTAG_NAMELIST},
	{"NECROMANCY",		 	DFNTAG_NECROMANCY},
	{"NEWBIE",			 	DFNTAG_NEWBIE},
	{"NINJITSU",			DFNTAG_NINJITSU},
	{"NOTRAIN",			 	DFNTAG_NOTRAIN},
	{"NPCAI",			 	DFNTAG_NPCAI},
	{"NPCLIST",			 	DFNTAG_NPCLIST},
	{"NPCWANDER",		 	DFNTAG_NPCWANDER},
	{"OFFSPELL",			DFNTAG_OFFSPELL},
	{"PACKITEM",			DFNTAG_PACKITEM},
	{"PARRYING",			DFNTAG_PARRYING},
	{"PEACEMAKING",		 	DFNTAG_PEACEMAKING},
	{"PILEABLE",			DFNTAG_PILEABLE},
	{"POLY",				DFNTAG_POLY},
	{"POISONDAMAGE",		 	DFNTAG_POISONDAMAGE},
	{"POISONSTRENGTH",	 	DFNTAG_POISONSTRENGTH},
	{"POISONED",			DFNTAG_POISONED},
	{"POISONING",		 	DFNTAG_POISONING},
	{"PRIV",				DFNTAG_PRIV},
	{"PROVOCATION",		 	DFNTAG_PROVOCATION},
	{"RACE",				DFNTAG_RACE},
	{"RAIN",				DFNTAG_RAIN},
	{"RANK",				DFNTAG_RANK},
	{"REATTACKAT",		 	DFNTAG_REATTACKAT},
	{"RESISTFIRE",		 	DFNTAG_RESISTFIRE},
	{"RESISTCOLD",		 	DFNTAG_RESISTCOLD},
	{"RESISTLIGHTNING",	 	DFNTAG_RESISTLIGHTNING},
	{"RESISTPOISON",		 	DFNTAG_RESISTPOISON},
	{"REMOVETRAPS",		 	DFNTAG_REMOVETRAPS},
	{"RESTOCK",			 	DFNTAG_RESTOCK},
	{"RSHOPITEM",		 	DFNTAG_RSHOPITEM},
	{"RUNS",				DFNTAG_RUNS},
	{"RUNNINGSPEED",		 	DFNTAG_RUNNINGSPEED},
	{"SAYCOLOR",			DFNTAG_SAYCOLOUR},
	{"SAYCOLOUR",		 	DFNTAG_SAYCOLOUR},
	{"SCRIPT",			 	DFNTAG_SCRIPT},
	{"SELLITEM",			DFNTAG_SELLITEM},
	{"SHOPITEM",			DFNTAG_SHOPITEM},
	{"SHOPKEEPER",		 	DFNTAG_SHOPKEEPER},
	{"SHOPLIST",			DFNTAG_SHOPLIST},
	{"SK_MADE",			 	DFNTAG_SK_MADE},
	{"SKILL",			 	DFNTAG_SKILL},
	{"SKIN",				DFNTAG_SKIN},
	{"SKINLIST",			DFNTAG_SKINLIST},
	{"SNOOPING",			DFNTAG_SNOOPING},
	{"SNOW",				DFNTAG_SNOW},
	{"SPADELAY",			DFNTAG_SPADELAY},
	{"SPATTACK",			DFNTAG_SPATTACK},
	{"SPAWNOBJ",			DFNTAG_SPAWNOBJ},
	{"SPAWNOBJLIST",		 	DFNTAG_SPAWNOBJLIST},
	{"SPD",				DFNTAG_SPD},
	{"SPEED",			 	DFNTAG_SPD},
	{"SPELLWEAVING",		 	DFNTAG_SPELLWEAVING},
	{"SPIRITSPEAK",		 	DFNTAG_SPIRITSPEAK},
	{"SPLIT",			 	DFNTAG_SPLIT},
	{"SPLITCHANCE",		 	DFNTAG_SPLITCHANCE},
	{"ST",				DFNTAG_STRENGTH},
	{"STAMINA",			 	DFNTAG_STAMINA},
	{"STAMINAMAX",		 	DFNTAG_STAMINAMAX},
	{"STR",				DFNTAG_STRENGTH},
	{"STRENGTH",			DFNTAG_STRENGTH},
	{"ST2",				DFNTAG_STRADD},
	{"STRADD",			 	DFNTAG_STRADD},
	{"STEALING",			DFNTAG_STEALING},
	{"STEALTH",			 	DFNTAG_STEALTH},
	{"SWORDSMANSHIP",	 		DFNTAG_SWORDSMANSHIP},
	{"TACTICS",			 	DFNTAG_TACTICS},
	{"TAILORING",		 	DFNTAG_TAILORING},
	{"TAMING",			 	DFNTAG_TAMING},
	{"TAMEDHUNGER",		 	DFNTAG_TAMEDHUNGER},
	{"TASTEID",			 	DFNTAG_TASTEID},
	{"THROWING",			DFNTAG_THROWING},
	{"TINKERING",		 	DFNTAG_TINKERING},
	{"TITLE",			 	DFNTAG_TITLE},
	{"TOTAME",			 	DFNTAG_TOTAME},
	{"TOPROV",			 	DFNTAG_TOPROV},
	{"TOPEACE",			 	DFNTAG_TOPEACE},
	{"TRACKING",			DFNTAG_TRACKING},
	{"TYPE",				DFNTAG_TYPE},
	{"VALUE",			 	DFNTAG_VALUE},
	{"VETERINARY",		 	DFNTAG_VETERINARY},
	{"VISIBLE",			 	DFNTAG_VISIBLE},
	{"WALKINGSPEED",		 	DFNTAG_WALKINGSPEED},
	{"WEIGHT",			 	DFNTAG_WEIGHT},
	{"WEIGHTMAX",		 	DFNTAG_WEIGHTMAX},
	{"WILLHUNGER",		 	DFNTAG_WILLHUNGER},
	{"WIPE",				DFNTAG_WIPE},
	{"WRESTLING",		 	DFNTAG_WRESTLING}
};

const std::map<DFNTAGS, KeyValueType> KeyValue::_keytype_mapping = {
	{DFNTAG_AC,				KeyValueType::numeric },
	{DFNTAG_ADVOBJ,			KeyValueType::numeric},
	{DFNTAG_ALCHEMY,			KeyValueType::doublenumeric },
	{DFNTAG_AMMO, 			KeyValueType::text },
	{DFNTAG_AMMOFX, 		 	KeyValueType::text },
	{DFNTAG_AMOUNT, 		 	KeyValueType::numeric },
	{DFNTAG_ANATOMY, 		 	KeyValueType::doublenumeric },
	{DFNTAG_ANIMALLORE,		KeyValueType::doublenumeric },
	{DFNTAG_ARCHERY, 		 	KeyValueType::doublenumeric },
	{DFNTAG_ARMSLORE, 		KeyValueType::doublenumeric },
	{DFNTAG_ATT, 			KeyValueType::doublenumeric },
	{DFNTAG_BACKPACK, 		KeyValueType::nodata },
	{DFNTAG_BASERANGE, 		KeyValueType::numeric },
	{DFNTAG_BEGGING, 		 	KeyValueType::doublenumeric },
	{DFNTAG_BLACKSMITHING,	 	KeyValueType::doublenumeric },
	{DFNTAG_BOWCRAFT,		 	KeyValueType::doublenumeric },
	{DFNTAG_BUSHIDO,		 	KeyValueType::doublenumeric },
	{DFNTAG_CAMPING,		 	KeyValueType::doublenumeric },
	{DFNTAG_CARPENTRY,		KeyValueType::doublenumeric },
	{DFNTAG_CARTOGRAPHY,		KeyValueType::doublenumeric },
	{DFNTAG_CARVE,			KeyValueType::numeric },
	{DFNTAG_CHIVALRY,		 	KeyValueType::doublenumeric },
	{DFNTAG_COLD,			KeyValueType::numeric },
	{DFNTAG_COLOUR,			KeyValueType::numeric },
	{DFNTAG_COLOURLIST,		KeyValueType::text },
	{DFNTAG_COLOURMATCHHAIR,	KeyValueType::nodata },
	{DFNTAG_COOKING,		 	KeyValueType::doublenumeric },
	{DFNTAG_CORPSE,			KeyValueType::numeric },
	{DFNTAG_CREATOR,		 	KeyValueType::numeric },
	{DFNTAG_CUSTOMSTRINGTAG,	KeyValueType::text },
	{DFNTAG_CUSTOMINTTAG,		KeyValueType::text },
	{DFNTAG_DAMAGE,			KeyValueType::doublenumeric },
	{DFNTAG_DECAY,			KeyValueType::numeric },
	{DFNTAG_DEF,			KeyValueType::doublenumeric },
	{DFNTAG_DETECTINGHIDDEN,	KeyValueType::doublenumeric },
	{DFNTAG_DEX,			KeyValueType::doublenumeric },
	{DFNTAG_DEXADD,			KeyValueType::numeric },
	{DFNTAG_DIR,			KeyValueType::text },
	{DFNTAG_DISPELLABLE,		KeyValueType::nodata },
	{DFNTAG_DISABLED,		 	KeyValueType::numeric },
	{DFNTAG_DOORFLAG,		 	KeyValueType::numeric },
	{DFNTAG_DYE,			KeyValueType::numeric },
	{DFNTAG_DYEBEARD,		 	KeyValueType::numeric },
	{DFNTAG_DYEHAIR,		 	KeyValueType::numeric },
	{DFNTAG_ELEMENTRESIST,	 	KeyValueType::text },
	{DFNTAG_EMOTECOLOUR,		KeyValueType::numeric },
	{DFNTAG_ENTICEMENT,		KeyValueType::doublenumeric },
	{DFNTAG_EQUIPITEM,		KeyValueType::uppertext },
	{DFNTAG_EVALUATINGINTEL,	KeyValueType::doublenumeric },
	{DFNTAG_FAME,			KeyValueType::numeric },
	{DFNTAG_FENCING,			KeyValueType::doublenumeric },
	{DFNTAG_FISHING,			KeyValueType::doublenumeric },
	{DFNTAG_FLAG,			KeyValueType::doublenumeric },
	{DFNTAG_FLEEAT,			KeyValueType::numeric },
	{DFNTAG_FLEEINGSPEED,		KeyValueType::text },
	{DFNTAG_FOCUS,			KeyValueType::doublenumeric },
	{DFNTAG_FORENSICS,		KeyValueType::doublenumeric },
	{DFNTAG_FX1,			KeyValueType::numeric },
	{DFNTAG_FX2,			KeyValueType::numeric },
	{DFNTAG_FY1,			KeyValueType::numeric },
	{DFNTAG_FY2,			KeyValueType::numeric },
	{DFNTAG_FZ1,			KeyValueType::numeric },
	{DFNTAG_FOOD,			KeyValueType::uppertext },
	{DFNTAG_GET,			KeyValueType::uppertext },
	{DFNTAG_GLOW,			KeyValueType::numeric },
	{DFNTAG_GLOWBC,			KeyValueType::numeric },
	{DFNTAG_GOLD,			KeyValueType::doublenumeric},
	{DFNTAG_GOOD,			KeyValueType::numeric },
	{DFNTAG_HAIRCOLOUR,		KeyValueType::text },
	{DFNTAG_HEALING,			KeyValueType::doublenumeric },
	{DFNTAG_HEAT,			KeyValueType::numeric },
	{DFNTAG_HERDING,			KeyValueType::doublenumeric },
	{DFNTAG_HIDAMAGE,			KeyValueType::numeric },
	{DFNTAG_HIDING,			KeyValueType::doublenumeric },
	{DFNTAG_HP,				KeyValueType::doublenumeric },
	{DFNTAG_HPMAX,			KeyValueType::doublenumeric },
	{DFNTAG_ID,				KeyValueType::numeric },
	{DFNTAG_IMBUING,			KeyValueType::doublenumeric },
	{DFNTAG_INTADD,			KeyValueType::numeric },
	{DFNTAG_INTELLIGENCE,		KeyValueType::doublenumeric },
	{DFNTAG_INTERVAL,			KeyValueType::doublenumeric },
	{DFNTAG_INSCRIPTION,		KeyValueType::doublenumeric },
	{DFNTAG_ITEMID,			KeyValueType::numeric },
	{DFNTAG_ITEMLIST,			KeyValueType::uppertext },
	{DFNTAG_KARMA,			KeyValueType::numeric },
	{DFNTAG_KILLHAIR,			KeyValueType::nodata },
	{DFNTAG_KILLBEARD,		KeyValueType::nodata },
	{DFNTAG_KILLPACK,			KeyValueType::nodata },
	{DFNTAG_LAYER,			KeyValueType::numeric },
	{DFNTAG_LIGHT,			KeyValueType::numeric },
	{DFNTAG_LIGHTNING,		KeyValueType::numeric },
	{DFNTAG_LOCKPICKING,		KeyValueType::numeric },
	{DFNTAG_LODAMAGE,			KeyValueType::numeric },
	{DFNTAG_LOOT,			KeyValueType::uppertext },
	{DFNTAG_LUMBERJACKING,		KeyValueType::doublenumeric },
	{DFNTAG_MACEFIGHTING,		KeyValueType::doublenumeric },
	{DFNTAG_MAGERY,			KeyValueType::doublenumeric },
	{DFNTAG_MAGICRESISTANCE,	KeyValueType::doublenumeric },
	{DFNTAG_MANA,			KeyValueType::doublenumeric },
	{DFNTAG_MANAMAX,			KeyValueType::doublenumeric },
	{DFNTAG_MAXHP,			KeyValueType::numeric },
	{DFNTAG_MAXITEMS,			KeyValueType::numeric },
	{DFNTAG_MAXRANGE,			KeyValueType::numeric },
	{DFNTAG_MEDITATION,		KeyValueType::doublenumeric },
	{DFNTAG_MINING,			KeyValueType::doublenumeric },
	{DFNTAG_MOVABLE,			KeyValueType::numeric },
	{DFNTAG_MORE,			KeyValueType::numeric },
	{DFNTAG_MORE2,			KeyValueType::numeric },
	{DFNTAG_MOREX,			KeyValueType::numeric },
	{DFNTAG_MOREY,			KeyValueType::numeric },
	{DFNTAG_MOREZ,			KeyValueType::numeric },
	{DFNTAG_MUSICIANSHIP,		KeyValueType::doublenumeric },
	{DFNTAG_MYSTICISM,		KeyValueType::doublenumeric },
	{DFNTAG_NAME,			KeyValueType::text },
	{DFNTAG_NAME2,			KeyValueType::text },
	{DFNTAG_NAMELIST,			KeyValueType::text },
	{DFNTAG_NECROMANCY,		KeyValueType::doublenumeric },
	{DFNTAG_NEWBIE,			KeyValueType::nodata },
	{DFNTAG_NINJITSU,			KeyValueType::doublenumeric },
	{DFNTAG_NOTRAIN,			KeyValueType::nodata },
	{DFNTAG_NPCAI,			KeyValueType::numeric },
	{DFNTAG_NPCLIST,			KeyValueType::uppertext },
	{DFNTAG_NPCWANDER,		KeyValueType::numeric },
	{DFNTAG_OFFSPELL,			KeyValueType::numeric },
	{DFNTAG_PACKITEM,			KeyValueType::text},
	{DFNTAG_PARRYING,			KeyValueType::doublenumeric },
	{DFNTAG_PEACEMAKING,		KeyValueType::doublenumeric },
	{DFNTAG_PILEABLE,			KeyValueType::numeric },
	{DFNTAG_POISONDAMAGE,		KeyValueType::numeric },
	{DFNTAG_POISONSTRENGTH,		KeyValueType::numeric },
	{DFNTAG_POISONED,			KeyValueType::numeric },
	{DFNTAG_POISONING,		KeyValueType::doublenumeric },
	{DFNTAG_POLY,			KeyValueType::numeric },
	{DFNTAG_PRIV,			KeyValueType::numeric },
	{DFNTAG_PROVOCATION,		KeyValueType::doublenumeric },
	{DFNTAG_RACE,			KeyValueType::numeric },
	{DFNTAG_RAIN,			KeyValueType::numeric },
	{DFNTAG_RANK,			KeyValueType::numeric },
	{DFNTAG_REATTACKAT,		KeyValueType::numeric },
	{DFNTAG_REMOVETRAPS,		KeyValueType::doublenumeric },
	{DFNTAG_RESTOCK,			KeyValueType::numeric },
	{DFNTAG_RESISTFIRE,		KeyValueType::doublenumeric },
	{DFNTAG_RESISTCOLD,		KeyValueType::doublenumeric },
	{DFNTAG_RESISTLIGHTNING,	KeyValueType::doublenumeric },
	{DFNTAG_RESISTPOISON,		KeyValueType::doublenumeric },
	{DFNTAG_RSHOPITEM,		KeyValueType::text },
	{DFNTAG_RUNNINGSPEED,		KeyValueType::text },
	{DFNTAG_RUNS,			KeyValueType::nodata },
	{DFNTAG_SAYCOLOUR,		KeyValueType::numeric},
	{DFNTAG_SCRIPT,			KeyValueType::text },
	{DFNTAG_SELLITEM,			KeyValueType::text },
	{DFNTAG_SHOPITEM,			KeyValueType::text },
	{DFNTAG_SHOPKEEPER,		KeyValueType::nodata },
	{DFNTAG_SHOPLIST,			KeyValueType::text },
	{DFNTAG_SK_MADE,			KeyValueType::numeric},
	{DFNTAG_SKILL,			KeyValueType::doublenumeric },
	{DFNTAG_SKILLVAR,			KeyValueType::unknown},
	{DFNTAG_SKIN,			KeyValueType::numeric},
	{DFNTAG_SKINLIST,			KeyValueType::text },
	{DFNTAG_SNOOPING,			KeyValueType::doublenumeric },
	{DFNTAG_SNOW,			KeyValueType::numeric },
	{DFNTAG_SPADELAY,			KeyValueType::numeric },
	{DFNTAG_SPATTACK,			KeyValueType::numeric },
	{DFNTAG_SPAWNOBJ,			KeyValueType::text },
	{DFNTAG_SPAWNOBJLIST,		KeyValueType::text },
	{DFNTAG_SPD,			KeyValueType::numeric },
	{DFNTAG_SPELLWEAVING,		KeyValueType::doublenumeric },
	{DFNTAG_SPIRITSPEAK,		KeyValueType::doublenumeric },
	{DFNTAG_SPLIT,			KeyValueType::numeric },
	{DFNTAG_SPLITCHANCE,		KeyValueType::numeric },
	{DFNTAG_STAMINA,			KeyValueType::doublenumeric },
	{DFNTAG_STAMINAMAX,		KeyValueType::doublenumeric },
	{DFNTAG_STRENGTH,			KeyValueType::doublenumeric },
	{DFNTAG_STRADD,			KeyValueType::numeric },
	{DFNTAG_STEALING,			KeyValueType::doublenumeric },
	{DFNTAG_STEALTH,			KeyValueType::doublenumeric },
	{DFNTAG_SWORDSMANSHIP,		KeyValueType::doublenumeric },
	{DFNTAG_TACTICS,			KeyValueType::doublenumeric },
	{DFNTAG_TAILORING,		KeyValueType::doublenumeric },
	{DFNTAG_TAMING,			KeyValueType::doublenumeric },
	{DFNTAG_TAMEDHUNGER,		KeyValueType::doublenumeric },
	{DFNTAG_TASTEID,			KeyValueType::doublenumeric },
	{DFNTAG_THROWING,			KeyValueType::doublenumeric },
	{DFNTAG_TINKERING,		KeyValueType::doublenumeric },
	{DFNTAG_TITLE,			KeyValueType::text },
	{DFNTAG_TOTAME,			KeyValueType::numeric },
	{DFNTAG_TOPROV,			KeyValueType::numeric },
	{DFNTAG_TOPEACE,			KeyValueType::doublenumeric },
	{DFNTAG_TRACKING,			KeyValueType::doublenumeric },
	{DFNTAG_TYPE,			KeyValueType::numeric },
	{DFNTAG_VALUE,			KeyValueType::doublenumeric },
	{DFNTAG_VETERINARY,		KeyValueType::doublenumeric },
	{DFNTAG_VISIBLE,			KeyValueType::numeric },
	{DFNTAG_WALKINGSPEED,		KeyValueType::text },
	{DFNTAG_WEIGHT,			KeyValueType::numeric },
	{DFNTAG_WEIGHTMAX,		KeyValueType::numeric },
	{DFNTAG_WILLHUNGER,		KeyValueType::numeric },
	{DFNTAG_WIPE,			KeyValueType::numeric },
	{DFNTAG_WRESTLING,		KeyValueType::doublenumeric }
	
};

//==============================================================================
KeyValue::KeyValue() {
	firstnumber = 0 ;
	seccondnumber = 0 ;
}
//==============================================================================
DFNTAGS KeyValue::lookupKeyEnum(const std::string &key){
	auto iter = KeyValue::_keyenum_mapping.find(key) ;
	if (iter != KeyValue::_keyenum_mapping.end()) {
		return iter->second ;
	}
	return DFNTAG_INVALIDTAG ;
}
//==============================================================================
KeyValueType KeyValue::lookupKeyType(DFNTAGS keyenum){
	auto iter = KeyValue::_keytype_mapping.find(keyenum) ;
	if (iter != KeyValue::_keytype_mapping.end()) {
		return iter->second ;
	}
	return KeyValueType::invalid ;

}
//==============================================================================
std::string KeyValue::description() const {
	return original_line;
}
//==============================================================================
std::string KeyValue::origin() const {
	return std::string("Line: ")+std::to_string(linenumber) + std::string(" from: ")+fileorigin;
}
