// Copyright © 2021 Charles Kerr. All rights reserved.
// Created on:  5/22/21

#ifndef KeyValue_hpp
#define KeyValue_hpp

#include <cstdint>
#include <string>
#include <map>

enum class KeyValueType {
	invalid=-1,numeric,text, uppertext,nodata,unknown,doublenumeric};

enum  DFNTAGS
{
	DFNTAG_INVALIDTAG = -1,
	DFNTAG_AC = 0,
	DFNTAG_ADDMENUITEM,
	DFNTAG_ADVOBJ,
	DFNTAG_ALCHEMY,
	DFNTAG_AMMO,
	DFNTAG_AMMOFX,
	DFNTAG_AMOUNT,
	DFNTAG_ANATOMY,
	DFNTAG_ANIMALLORE,
	DFNTAG_ARCHERY,
	DFNTAG_ARMSLORE,
	DFNTAG_ATT,
	DFNTAG_BACKPACK,
	DFNTAG_BASERANGE,
	DFNTAG_BEGGING,
	DFNTAG_BLACKSMITHING,
	DFNTAG_BOWCRAFT,
	DFNTAG_BUSHIDO,
	DFNTAG_CAMPING,
	DFNTAG_CARPENTRY,
	DFNTAG_CARTOGRAPHY,
	DFNTAG_CARVE,
	DFNTAG_CHIVALRY,
	DFNTAG_COLD,
	DFNTAG_COLOUR,
	DFNTAG_COLOURLIST,
	DFNTAG_COLOURMATCHHAIR,
	DFNTAG_COOKING,
	DFNTAG_CORPSE,
	DFNTAG_CREATOR,
	DFNTAG_CUSTOMSTRINGTAG,
	DFNTAG_CUSTOMINTTAG,
	DFNTAG_DAMAGE,
	DFNTAG_DECAY,
	DFNTAG_DEF,
	DFNTAG_DETECTINGHIDDEN,
	DFNTAG_DEX,
	DFNTAG_DEXADD,
	DFNTAG_DIR,
	DFNTAG_DISPELLABLE,
	DFNTAG_DISABLED,
	DFNTAG_DOORFLAG,
	DFNTAG_DYE,
	DFNTAG_DYEBEARD,
	DFNTAG_DYEHAIR,
	DFNTAG_ELEMENTRESIST,
	DFNTAG_EMOTECOLOUR,
	DFNTAG_ENTICEMENT,
	DFNTAG_EQUIPITEM,
	DFNTAG_EVALUATINGINTEL,
	DFNTAG_FAME,
	DFNTAG_FENCING,
	DFNTAG_FISHING,
	DFNTAG_FLAG,
	DFNTAG_FLEEAT,
	DFNTAG_FLEEINGSPEED,
	DFNTAG_FOCUS,
	DFNTAG_FORENSICS,
	DFNTAG_FX1,
	DFNTAG_FX2,
	DFNTAG_FY1,
	DFNTAG_FY2,
	DFNTAG_FZ1,
	DFNTAG_FOOD,
	DFNTAG_GET,
	DFNTAG_GLOW,
	DFNTAG_GLOWBC,
	DFNTAG_GLOWTYPE,
	DFNTAG_GOLD,
	DFNTAG_GOOD,
	DFNTAG_HAIRCOLOUR,
	DFNTAG_HEALING,
	DFNTAG_HEAT,
	DFNTAG_HERDING,
	DFNTAG_HIDAMAGE,
	DFNTAG_HIDING,
	DFNTAG_HP,
	DFNTAG_HPMAX,
	DFNTAG_ID,
	DFNTAG_IMBUING,
	DFNTAG_INTADD,
	DFNTAG_INTELLIGENCE,
	DFNTAG_INTERVAL,
	DFNTAG_INSCRIPTION,
	DFNTAG_ITEMID,
	DFNTAG_ITEMLIST,
	DFNTAG_KARMA,
	DFNTAG_KILLHAIR,
	DFNTAG_KILLBEARD,
	DFNTAG_KILLPACK,
	DFNTAG_LAYER,
	DFNTAG_LIGHT,
	DFNTAG_LIGHTNING,
	DFNTAG_LOCKPICKING,
	DFNTAG_LODAMAGE,
	DFNTAG_LOOT,
	DFNTAG_LUMBERJACKING,
	DFNTAG_MACEFIGHTING,
	DFNTAG_MAGERY,
	DFNTAG_MAGICRESISTANCE,
	DFNTAG_MANA,
	DFNTAG_MANAMAX,
	DFNTAG_MAXHP,
	DFNTAG_MAXITEMS,
	DFNTAG_MAXRANGE,
	DFNTAG_MEDITATION,
	DFNTAG_MINING,
	DFNTAG_MOVABLE,
	DFNTAG_MORE,
	DFNTAG_MORE2,
	DFNTAG_MOREX,
	DFNTAG_MOREY,
	DFNTAG_MOREZ,
	DFNTAG_MUSICIANSHIP,
	DFNTAG_MYSTICISM,
	DFNTAG_NAME,
	DFNTAG_NAME2,
	DFNTAG_NAMELIST,
	DFNTAG_NECROMANCY,
	DFNTAG_NEWBIE,
	DFNTAG_NINJITSU,
	DFNTAG_NOTRAIN,
	DFNTAG_NPCAI,
	DFNTAG_NPCLIST,
	DFNTAG_NPCWANDER,
	DFNTAG_OFFSPELL,
	DFNTAG_PACKITEM,
	DFNTAG_PARRYING,
	DFNTAG_PEACEMAKING,
	DFNTAG_PILEABLE,
	DFNTAG_POISONDAMAGE,
	DFNTAG_POISONSTRENGTH,
	DFNTAG_POISONED,
	DFNTAG_POISONING,
	DFNTAG_POLY,
	DFNTAG_PRIV,
	DFNTAG_PROVOCATION,
	DFNTAG_RACE,
	DFNTAG_RAIN,
	DFNTAG_RANK,
	DFNTAG_REATTACKAT,
	DFNTAG_REMOVETRAPS,
	DFNTAG_RESTOCK,
	DFNTAG_RESISTFIRE,
	DFNTAG_RESISTCOLD,
	DFNTAG_RESISTLIGHTNING,
	DFNTAG_RESISTPOISON,
	DFNTAG_RSHOPITEM,
	DFNTAG_RUNNINGSPEED,
	DFNTAG_RUNS,
	DFNTAG_SAYCOLOUR,
	DFNTAG_SCRIPT,
	DFNTAG_SELLITEM,
	DFNTAG_SHOPITEM,
	DFNTAG_SHOPKEEPER,
	DFNTAG_SHOPLIST,
	DFNTAG_SK_MADE,
	DFNTAG_SKILL,
	DFNTAG_SKILLVAR,
	DFNTAG_SKIN,
	DFNTAG_SKINLIST,
	DFNTAG_SNOOPING,
	DFNTAG_SNOW,
	DFNTAG_SPADELAY,
	DFNTAG_SPATTACK,
	DFNTAG_SPAWNOBJ,
	DFNTAG_SPAWNOBJLIST,
	DFNTAG_SPD,
	DFNTAG_SPELLWEAVING,
	DFNTAG_SPIRITSPEAK,
	DFNTAG_SPLIT,
	DFNTAG_SPLITCHANCE,
	DFNTAG_STAMINA,
	DFNTAG_STAMINAMAX,
	DFNTAG_STRENGTH,
	DFNTAG_STRADD,
	DFNTAG_STEALING,
	DFNTAG_STEALTH,
	DFNTAG_SWORDSMANSHIP,
	DFNTAG_TACTICS,
	DFNTAG_TAILORING,
	DFNTAG_TAMING,
	DFNTAG_TAMEDHUNGER,
	DFNTAG_TASTEID,
	DFNTAG_THROWING,
	DFNTAG_TINKERING,
	DFNTAG_TITLE,
	DFNTAG_TOTAME,
	DFNTAG_TOPROV,
	DFNTAG_TOPEACE,
	DFNTAG_TRACKING,
	DFNTAG_TYPE,
	DFNTAG_VALUE,
	DFNTAG_VETERINARY,
	DFNTAG_VISIBLE,
	DFNTAG_WALKINGSPEED,
	DFNTAG_WEIGHT,
	DFNTAG_WEIGHTMAX,
	DFNTAG_WILLHUNGER,
	DFNTAG_WIPE,
	DFNTAG_WRESTLING
};

struct KeyValue {
	static const std::map<std::string, DFNTAGS> _keyenum_mapping ;
	static const std::map<DFNTAGS, KeyValueType> _keytype_mapping ;

	std::string fileorigin;
	int linenumber ;
	
	std::string original_line ;
	
	std::string key ;
	std::string ukey ;
	std::string value ;
	
	DFNTAGS etag ;
	std::string tag ;
	
	std::string textdata ;
	int firstnumber ;
	int seccondnumber ;
	
	KeyValue();
	std::string description() const ;
	std::string origin() const ;
	static DFNTAGS lookupKeyEnum(const std::string &key);
	static KeyValueType lookupKeyType(DFNTAGS keyenum);
};

#endif /* KeyValue_hpp */
